/*
 * This file was generated by dbusxml2cpp version 0.6
 * Command line was: dbusxml2cpp -m -p avahi_serviceresolver_interface /home/qba/src/kdelibs/dnssd/org.freedesktop.Avahi.ServiceResolver.xml
 *
 * dbusxml2cpp is Copyright (C) 2006 Trolltech ASA. All rights reserved.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef AVAHI_SERVICERESOLVER_INTERFACE_H_1175536773
#define AVAHI_SERVICERESOLVER_INTERFACE_H_1175536773

#include <QDBusAbstractInterface>
#include <QDBusConnection>
#include <QDBusReply>
#include <QList>
#include <QMap>
#include <QObject>
#include <QString>
#include <QStringList>
#include <QVariant>

/*
 * Proxy class for interface org.freedesktop.Avahi.ServiceResolver
 */
class OrgFreedesktopAvahiServiceResolverInterface : public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    {
        return "org.freedesktop.Avahi.ServiceResolver";
    }

public:
    OrgFreedesktopAvahiServiceResolverInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgFreedesktopAvahiServiceResolverInterface() override;

public Q_SLOTS: // METHODS
    inline QDBusReply<void> Free()
    {
        QList<QVariant> argumentList;
        return callWithArgumentList(QDBus::Block, QLatin1String("Free"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void Failure(const QString &error);
    void Found(int interface,
               int protocol,
               const QString &name,
               const QString &type,
               const QString &domain,
               const QString &host,
               int aprotocol,
               const QString &address,
               ushort port,
               const QList<QByteArray> &txt,
               uint flags);
};

namespace org
{
namespace freedesktop
{
namespace Avahi
{
typedef ::OrgFreedesktopAvahiServiceResolverInterface ServiceResolver;
}
}
}
#endif
