/*
    This file is part of the KDE project

    SPDX-FileCopyrightText: 2018 Harald Sitter <sitter@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "avahi_listener_p.h"

namespace KDNSSD
{
AvahiListener::AvahiListener()
{
}

AvahiListener::~AvahiListener()
{
}

} // namespace KDNSSD
